# -*- coding: utf-8 -*-
from collective.easyform import easyformMessageFactory as _
from collective.easyform.api import get_expression
from collective.easyform.interfaces import IEasyForm
from collective.easyform.interfaces import IEasyFormForm
from collective.easyform.interfaces import IFieldExtender
from collective.easyform.interfaces import ILabel
from collective.easyform.interfaces import IHCaptcha
from collective.easyform.interfaces import INorobotCaptcha
from collective.easyform.interfaces import IReCaptcha
from collective.easyform.interfaces import ILikert
from collective.easyform.interfaces import IRichLabel
from collective.easyform.validators import IFieldValidator
from plone.schemaeditor.fields import FieldFactory
from plone.supermodel.exportimport import BaseHandler
from z3c.form.interfaces import IGroup
from z3c.form.interfaces import IValidator
from z3c.form.interfaces import IValue
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import Invalid
from zope.interface import providedBy
from zope.schema import Field
from zope.schema import TextLine
from zope.schema._bootstrapinterfaces import IFromUnicode
from zope.schema.interfaces import IField


def superAdapter(specific_interface, adapter, objects, name=u""):
    """Find the next most specific adapter.

    This is called by a FieldExtenderValidator or FieldExtenderDefault instance.
    This is passed in with the 'adapter' parameter.
    This adapter itself is not a real validator or default factory,
    but is used to find other real validators or default factories.

    This may sound strange, but it solves a problem.
    Problem is that validators and default were not always found for fields in field sets.
    For example, a captcha field in the main form would get validated by its proper validator,
    but when in a field set, only a basic validator would be found.

    We are adjusting the view object class to provide IForm rather than
    IEasyFormForm or IGroup to make one of the objects less specific.
    Same for the default factory.
    This allows us to find another adapter other than the current one.
    This allows us to find any custom adapters for any fields that we have overridden

    """
    new_obj = []
    found = False
    for obj in objects:
        interfaces = list(providedBy(obj).interfaces())
        try:
            index = interfaces.index(specific_interface)
            found = True
        except ValueError:
            pass
        else:
            super_inferface = interfaces[index + 1]

            @implementer(super_inferface)
            class Wrapper(object):
                def __init__(self, view):
                    self.__view__ = view

                def __getattr__(self, item):
                    return getattr(self.__view__, item)

            obj = Wrapper(obj)  # Make one class less specific
        new_obj.append(obj)
    if not found:
        return None

    provided_by = providedBy(adapter)
    # With zope.interface 5.0.2, the info we seek is in 'declared'.
    # With 5.1.0+, it can also be in the 'interfaces()' iterator,
    # especially for groups (field sets).
    # But it looks like interfaces() works always.
    # adapter_interfaces = provided_by.declared
    # if not adapter_interfaces:
    adapter_interfaces = list(provided_by.interfaces())
    if not adapter_interfaces:
        return

    return queryMultiAdapter(new_obj, adapter_interfaces[0], name=name)


@implementer(IValidator)
@adapter(IEasyForm, Interface, IEasyFormForm, IField, Interface)
class FieldExtenderValidator(object):
    """z3c.form validator class for easyform fields in the default fieldset"""

    def __init__(self, context, request, view, field, widget):
        self.context = context
        self.request = request
        self.view = view
        self.field = field
        self.widget = widget

    def validate(self, value):
        """Validate field by TValidator"""
        # By default this will call SimpleFieldValidator.validator but allows for a fields
        # custom validation adaptor to also be called such as recaptcha
        _, _, view_interface, _, _ = self.__class__.__component_adapts__
        validator = superAdapter(
            view_interface,
            self,
            (self.context, self.request, self.view, self.field, self.widget),
        )
        if validator is not None:
            validator.validate(value)

        efield = IFieldExtender(self.field)
        validators = getattr(efield, "validators", [])
        if validators:
            for validator in validators:
                vmethod = queryUtility(IFieldValidator, name=validator)
                if not vmethod:
                    continue
                res = vmethod(value, REQUEST=self.request)
                if res:
                    raise Invalid(res)
        TValidator = getattr(efield, "TValidator", None)
        if TValidator:
            try:
                cerr = get_expression(self.context, TValidator, value=value)
            except Exception as e:
                raise Invalid(e)
            if cerr:
                raise Invalid(cerr)


@implementer(IValidator)
@adapter(IEasyForm, Interface, IGroup, IField, Interface)
class GroupFieldExtenderValidator(FieldExtenderValidator):
    """z3c.form validator class for easyform fields in fieldset groups"""

    pass


@implementer(IValue)
@adapter(IEasyForm, Interface, IEasyFormForm, IField, Interface)
class FieldExtenderDefault(object):
    """z3c.form default class for easyform fields in the default fieldset"""

    def __init__(self, context, request, view, field, widget):
        self.context = context
        self.request = request
        self.view = view
        self.field = field
        self.widget = widget

    def get(self):
        """Get default value of field from TDefault"""
        efield = IFieldExtender(self.field)
        TDefault = getattr(efield, "TDefault", None)
        if TDefault:
            return get_expression(self.context, TDefault)

        # see if there is another default adapter for this field instead
        _, _, view_interface, _, _ = self.__class__.__component_adapts__
        adapter = superAdapter(
            view_interface,
            self,
            (self.context, self.request, self.view, self.field, self.widget),
            name="default",
        )
        if adapter is not None:
            return adapter.get()
        else:
            # TODO: this should have already been done by z3c.form.widget.update() so shouldn't be needed
            return self.field.default


@implementer(IValue)
@adapter(IEasyForm, Interface, IGroup, IField, Interface)
class GroupFieldExtenderDefault(FieldExtenderDefault):
    """z3c.form default class for easyform fields in fieldset groups"""

    pass


@implementer(IFromUnicode, ILabel)
class Label(Field):
    """A Label field"""

    def validate(self, value):
        pass

    def fromUnicode(self, str):
        """ """
        return


@implementer(IRichLabel)
class RichLabel(Label):
    """A Rich Label field"""

    rich_label = u""

    def __init__(self, rich_label=u"", **kw):
        self.rich_label = rich_label
        super(RichLabel, self).__init__(**kw)


LabelFactory = FieldFactory(Label, _(u"label_label_field", default=u"Label"))
RichLabelFactory = FieldFactory(
    RichLabel, _(u"label_richlabel_field", default=u"Rich Label")
)

LabelHandler = BaseHandler(Label)
RichLabelHandler = BaseHandler(RichLabel)


@implementer(IReCaptcha)
class ReCaptcha(TextLine):
    """A ReCaptcha field"""


ReCaptchaFactory = FieldFactory(
    ReCaptcha, _(u"label_recaptcha_field", default=u"ReCaptcha")
)
ReCaptchaHandler = BaseHandler(ReCaptcha)

@implementer(IHCaptcha)
class HCaptcha(TextLine):
    """A HCaptcha field"""


HCaptchaFactory = FieldFactory(
    HCaptcha, _(u"label_hcaptcha_field", default=u"HCaptcha")
)
HCaptchaHandler = BaseHandler(HCaptcha)


@implementer(INorobotCaptcha)
class NorobotCaptcha(TextLine):
    """A NorobotCaptcha field"""


NorobotFactory = FieldFactory(
    NorobotCaptcha, _(u"label_norobot_field", default=u"NorobotCaptcha")
)
NorobotCaptchaHandler = BaseHandler(NorobotCaptcha)


@implementer(ILikert)
class Likert(TextLine):
    """A Likert field"""

    def __init__(self, **kwargs):
        self.answers = kwargs.get('answers', None)
        if 'answers' in kwargs:
            del kwargs['answers']
        self.questions = kwargs.get('questions', None)
        if 'questions' in kwargs:
            del kwargs['questions']
        Field.__init__(self, **kwargs)

    def _validate(self, value):
        super(Likert, self)._validate(value)
        self.parse(value)

    def parse(self, value):
        result = dict()
        lines = value.split(',')
        for line in lines:
            if not line:
                continue
            id, answer = line.split(':')
            answer = answer.strip()
            if answer not in self.answers:
                raise ValueError('Invalid answer value.')
            index = int(id)
            if index < 1 or index > len(self.questions):
                raise ValueError('Invalid question index.')
            result[index] = answer
        return result


LikertFactory = FieldFactory(
    Likert, _(u"label_likert_field", default=u"Likert")
)
LikertHandler = BaseHandler(Likert)




# -*- coding: utf-8 -*-
#
# Integeration tests specific to the mailer
#

from collective.easyform.browser.view import EasyFormForm
from collective.easyform.tests import base
from os.path import dirname
from os.path import join
from plone import api
from plone.namedfile.file import NamedFile


class TestFunctions(base.EasyFormTestCase):
    """Test mailer action"""

    def afterSetUp(self):
        super(TestFunctions, self).afterSetUp()
        ff1_id = self.folder.invokeFactory("EasyForm", "ff1")
        self.ff1 = self.folder[ff1_id]

    def LoadRequestForm(self, **kwargs):
        request = self.layer["request"]
        request.form.clear()
        prefix = "form.widgets."
        for key in kwargs.keys():
            request.form[prefix + key] = kwargs[key]
        return request

    def test_thankspage(self):
        """Test thankspage"""

        data = {
            "topic": "test subject",
            "comments": "test comments",
            "replyto": "foo@bar.com",
        }
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertIn("Thanks for your input.", form)
        self.assertIn(
            '<span id="form-widgets-replyto" class="text-widget required textline-field">foo@bar.com</span>',  # noqa
            form,
        )

    def test_thankspage_filter(self):
        """Test thankspage"""
        self.ff1.showAll = False
        self.ff1.showFields = ["comments"]
        data = {
            "topic": "test subject",
            "comments": "test comments",
            "replyto": "foo@bar.com",
        }
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertIn("Thanks for your input.", form)
        self.assertNotIn("subject", form)
        self.assertIn("test comments", form)

    def test_thankspage_radio(self):
        self.ff1.showAll = True
        field_template = api.content.create(
            self.layer["portal"], "File", id="easyform_default_fields.xml"
        )
        with open(join(dirname(__file__), "fixtures", "radio_form.xml")) as f:
            filecontent = NamedFile(f.read(), contentType="application/xml")
        field_template.file = filecontent

        data = {"yes": "true", "yes-empty-marker": 1}
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertIn(
            '<span id="form-widgets-yes" class="radio-widget bool-field"><span class="selected-option">yes</span></span>',  # noqa
            form,
        )

    def test_show_hidden_on_thankspage(self):
        self.ff1.showAll = True
        field_template = api.content.create(
            self.layer["portal"], "File", id="easyform_default_fields.xml"
        )
        with open(join(dirname(__file__), "fixtures", "hidden_form.xml")) as f:
            filecontent = NamedFile(f.read(), contentType="application/xml")
        field_template.file = filecontent

        data = {"hide": "hello hidden"}
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertIn(
            '<span id="form-widgets-hide" class="text-widget textline-field">hello hidden</span>',  # noqa
            form,
        )

    def test_no_widget_on_thankspage_fieldset(self):
        self.ff1.showAll = True
        field_template = api.content.create(
            self.layer["portal"], "File", id="easyform_default_fields.xml"
        )
        with open(join(dirname(__file__), "fixtures", "fieldset_form.xml")) as f:
            filecontent = NamedFile(f.read(), contentType="application/xml")
        field_template.file = filecontent

        data = {
            "first-field": "hello ff",
            "second-field-one": "hello sf1",
            "second-field-two": "hello sf2",
            "second-field-three": "hello sf3",
        }
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertNotIn('<input id="form-widgets-second-field-one"', form)
        self.assertNotIn('<input id="form-widgets-second-field-two"', form)
        self.assertNotIn('<input id="form-widgets-second-field-three"', form)

    def test_not_showall_on_thankspage_fieldset(self):
        self.ff1.showAll = False
        field_template = api.content.create(
            self.layer["portal"], "File", id="easyform_default_fields.xml"
        )
        with open(join(dirname(__file__), "fixtures", "fieldset_form.xml")) as f:
            filecontent = NamedFile(f.read(), contentType="application/xml")
        field_template.file = filecontent

        data = {
            "first-field": "hello ff",
            "second-field-one": "hello sf1",
            "second-field-two": "hello sf2",
            "second-field-three": "hello sf3",
        }
        request = self.LoadRequestForm(**data)
        request.method = "POST"
        form = EasyFormForm(self.ff1, request)()
        self.assertNotIn('<input id="form-widgets-second-field-one"', form)
        self.assertNotIn('<input id="form-widgets-second-field-two"', form)
        self.assertNotIn('<input id="form-widgets-second-field-three"', form)


class TestThanksPageTraverseFunctional(base.EasyFormFunctionalTestCase):
    def setUp(self):
        # TODO...
        pass

    def test_redirect_to_thankspage(self):
        return
        # TODO...
        portal_url = self.portal.absolute_url()
        self.portal.invokeFactory("Folder", "news")
        self.browser.open(portal_url + "/testform/actions/mailer")
        self.browser.getControl(
            name="form.widgets.recipient_email"
        ).value = "mdummy@address.com"
        self.browser.getControl("Save").click()
        self.browser.open(portal_url + "/testform/edit")
        self.browser.getControl("Traverse to").selected = True
        self.browser.getControl(
            name="form.widgets.thanksPageOverride"
        ).value = "string:news"
        self.browser.getControl("Save").click()
        self.browser.getControl("Your E-Mail Address").value = "test@example.com"
        self.browser.getControl("Subject").value = "Test Subject"
        self.browser.getControl("Comments").value = "PFG rocks!"
        self.browser.getControl("Submit").click()
        self.assertIn("Test Subject", self.browser.contents)
        self.assertIn("PFG rocks!", self.browser.contents)
        self.assertNotIn("Thanks for your input.", self.browser.contents)
        self.assertEqual(self.browser.url, "http://nohost/plone/testform")
        self.assertNotIn("Thanks for your input.", self.browser.contents)



# -*- coding: utf-8 -*-
from .validators import isTALES
from collective.easyform import config
from collective.easyform import easyformMessageFactory as _  # NOQA
from plone import api
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.supermodel.directives import fieldset
from plone.supermodel.model import Schema
from Products.CMFPlone.utils import safe_unicode
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import provider

import zope.i18nmessageid
import zope.interface
import zope.schema.interfaces


PMF = zope.i18nmessageid.MessageFactory("plone")


@provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_submitLabel(context):
    return translate(
        _(u"default_submitLabel", u"Submit"),
        target_language=api.portal.get_current_language(),
    )


@provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_resetLabel(context):
    return translate(
        _(u"default_resetLabel", u"Reset"),
        target_language=api.portal.get_current_language(),
    )


@provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_thankstitle(context):
    return translate(
        _(u"default_thankstitle", u"Thank You"),
        target_language=api.portal.get_current_language(),
    )


@provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_thanksdescription(context):
    return translate(
        _(u"default_thanksdescription", u"Thanks for your input."),
        target_language=api.portal.get_current_language(),
    )


@zope.interface.provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_actions(context):
    """Default mail body for mailer action.
    Acquire 'mail_body_default.pt' or return hard coded default
    """
    portal = api.portal.get()
    default_actions = portal.restrictedTraverse(
        "easyform_default_actions.xml", default=None
    )
    if default_actions:
        return safe_unicode(default_actions.file.data)
    else:
        return config.ACTIONS_DEFAULT


@zope.interface.provider(zope.schema.interfaces.IContextAwareDefaultFactory)
def default_fields(context):
    """Default form fields.
    Acquire 'easyform_default_fields.xml' or return hard coded default
    """
    portal = api.portal.get()
    default_fields = portal.restrictedTraverse(
        "easyform_default_fields.xml", default=None
    )
    if default_fields:
        return safe_unicode(default_fields.file.data)
    else:
        return config.FIELDS_DEFAULT


class IEasyForm(Schema):
    """Forms for Plone."""

    directives.omitted("fields_model", "actions_model")
    fields_model = zope.schema.Text(
        title=_(u"Fields Model"), defaultFactory=default_fields
    )
    actions_model = zope.schema.Text(
        title=_(u"Actions Model"), defaultFactory=default_actions
    )

    # DEFAULT
    formPrologue = RichText(
        title=_(u"label_prologue_text", default=u"Form Prologue"),
        description=_(
            u"help_prologue_text",
            default=u"This text will be displayed above the form fields.",
        ),
        required=False,
    )
    formEpilogue = RichText(
        title=_(u"label_epilogue_text", default=u"Form Epilogue"),
        description=_(
            u"help_epilogue_text",
            default=u"The text will be displayed after the form fields.",
        ),
        required=False,
    )

    # THANKYOU
    fieldset(
        u"thankyou",
        label=_("Thanks Page"),
        fields=[
            "thankstitle",
            "thanksdescription",
            "showAll",
            "showFields",
            "includeEmpties",
            "thanksPrologue",
            "thanksEpilogue",
        ],
        order=10,
    )
    thankstitle = zope.schema.TextLine(
        title=_(u"label_thankstitle", default=u"Thanks title"),
        defaultFactory=default_thankstitle,
        required=True,
    )
    thanksdescription = zope.schema.Text(
        title=_(u"label_thanksdescription", default=u"Thanks summary"),
        description=_(u"help_thanksdescription", default=u"Used in thanks page."),
        defaultFactory=default_thanksdescription,
        required=False,
        missing_value=u"",
    )
    showAll = zope.schema.Bool(
        title=_(u"label_showallfields_text", default=u"Show All Fields"),
        description=_(
            u"help_showallfields_text",
            default=u""
            u"Check this to display input for all fields "
            u"(except label and file fields). If you check "
            u"this, the choices in the pick box below "
            u"will be ignored.",
        ),
        default=True,
        required=False,
    )
    showFields = zope.schema.List(
        title=_(u"label_showfields_text", default=u"Show Responses"),
        description=_(
            u"help_showfields_text",
            default=u"Pick the fields whose inputs you'd like to display on "
            u"the success page.",
        ),
        unique=True,
        required=False,
        value_type=zope.schema.Choice(vocabulary="easyform.Fields"),  # noqa
    )
    includeEmpties = zope.schema.Bool(
        title=_(u"label_includeEmpties_text", default=u"Include Empties"),
        description=_(
            u"help_includeEmpties_text",
            default=u""
            u"Check this to display field titles "
            u"for fields that received no input. Uncheck "
            u"to leave fields with no input off the list.",
        ),
        default=True,
        required=False,
    )
    thanksPrologue = RichText(
        title=_(u"label_thanksprologue_text", default=u"Thanks Prologue"),
        description=_(
            u"help_thanksprologue_text",
            default=u"This text will be displayed above the selected field " u"inputs.",
        ),
        required=False,
    )
    thanksEpilogue = RichText(
        title=_(u"label_thanksepilogue_text", default=u"Thanks Epilogue"),
        description=_(
            u"help_thanksepilogue_text",
            default=u"The text will be displayed after the field inputs.",
        ),
        required=False,
    )

    # ADVANCED
    fieldset(
        u"advanced",
        label=_("Advanced"),
        fields=[
            "submitLabel",
            "useCancelButton",
            "resetLabel",
            "form_tabbing",
            "nameAttribute",
            "default_fieldset_label",
            "method",
            "unload_protection",
            "CSRFProtection",
            "forceSSL",
        ],
        order=20,
    )
    submitLabel = zope.schema.TextLine(
        title=_(u"label_submitlabel_text", default=u"Submit Button Label"),
        description=_(u"help_submitlabel_text", default=u""),
        defaultFactory=default_submitLabel,
        required=False,
    )
    useCancelButton = zope.schema.Bool(
        title=_(u"label_showcancel_text", default=u"Show Reset Button"),
        description=_(u"help_showcancel_text", default=u""),
        default=False,
        required=False,
    )
    resetLabel = zope.schema.TextLine(
        title=_(u"label_reset_button", default=u"Reset Button Label"),
        description=_(u"help_reset_button", default=u""),
        defaultFactory=default_resetLabel,
        required=False,
    )
    nameAttribute = zope.schema.TextLine(
        title=_(u"label_name_attribute", default=u"Name attribute"),
        description=_(
            u"help_name_attribute",
            default=u"optional, sets the name attribute on the form container. " \
                    u"can be used for form analytics"),
        required=False,
    )
    directives.write_permission(form_tabbing=config.EDIT_ADVANCED_PERMISSION)
    form_tabbing = zope.schema.Bool(
        title=_(u"label_form_tabbing", default=u"Turn fieldsets to tabs"),
        description=_(u"help_form_tabbing", default=u""),
        default=True,
        required=False,
    )
    directives.write_permission(default_fieldset_label=config.EDIT_ADVANCED_PERMISSION)
    default_fieldset_label = zope.schema.TextLine(
        title=_(
            u"label_default_fieldset_label_text",
            default=u"Custom Default Fieldset Label",
        ),
        description=_(
            u"help_default_fieldset_label_text",
            default=u"This field allows you to change default fieldset label.",
        ),
        required=False,
        default=u"",
    )
    directives.write_permission(method=config.EDIT_TECHNICAL_PERMISSION)
    method = zope.schema.Choice(
        title=_(u"label_method", default=u"Form method"),
        description=_(u"help_method", default=u""),
        default=u"post",
        required=False,
        vocabulary="easyform.FormMethods",
    )
    directives.write_permission(unload_protection=config.EDIT_TECHNICAL_PERMISSION)
    unload_protection = zope.schema.Bool(
        title=_(u"label_unload_protection", default=u"Unload protection"),
        description=_(u"help_unload_protection", default=u""),
        default=True,
        required=False,
    )
    directives.write_permission(CSRFProtection=config.EDIT_TECHNICAL_PERMISSION)
    CSRFProtection = zope.schema.Bool(
        title=_(u"label_csrf", default=u"CSRF Protection"),
        description=_(
            u"help_csrf",
            default=u"Check this to employ Cross-Site "
            u"Request Forgery protection. Note that only HTTP Post "
            u"actions will be allowed.",
        ),
        default=True,
        required=False,
    )
    directives.write_permission(forceSSL=config.EDIT_TECHNICAL_PERMISSION)
    forceSSL = zope.schema.Bool(
        title=_(u"label_force_ssl", default=u"Force SSL connection"),
        description=_(
            u"help_force_ssl",
            default=u"Check this to make the form redirect to an SSL-enabled "
            u"version of itself (https://) if accessed via a non-SSL "
            u"URL (http://).  In order to function properly, "
            u"this requires a web server that has been configured to "
            u"handle the HTTPS protocol on port 443 and forward it to "
            u"Zope.",
        ),
        default=False,
        required=False,
    )

    # OVERRIDES
    fieldset(
        u"overrides",
        label=_("Overrides"),
        fields=[
            "thanksPageOverrideAction",
            "thanksPageOverride",
            "formActionOverride",
            "onDisplayOverride",
            "afterValidationOverride",
            "headerInjection",
            "submitLabelOverride",
        ],
        order=30,
    )
    directives.write_permission(thanksPageOverrideAction=config.EDIT_TALES_PERMISSION)
    thanksPageOverrideAction = zope.schema.Choice(
        title=_(
            u"label_thankspageoverrideaction_text",
            default=u"Custom Success Action Type",
        ),
        description=_(
            u"help_thankspageoverrideaction_text",
            default=u"Use this field in place of a thanks-page designation "
            u"to determine final action after calling "
            u"your action adapter (if you have one). You would "
            u"usually use this for a custom success template or "
            u"script. Leave empty if unneeded. Otherwise, specify as "
            u"you would a CMFFormController action type and argument, "
            u"complete with type of action to execute "
            u'(e.g., "redirect_to" or "traverse_to") '
            u"and a TALES expression. For example, "
            u'"Redirect to" and "string:thanks-page" would redirect '
            u'to "thanks-page".',
        ),
        default=u"redirect_to",
        required=False,
        vocabulary="easyform.CustomActions",
    )
    directives.write_permission(thanksPageOverride=config.EDIT_TALES_PERMISSION)
    thanksPageOverride = zope.schema.TextLine(
        title=_(u"label_thankspageoverride_text", default=u"Custom Success Action"),
        description=_(
            u"help_thankspageoverride_text",
            default=u"Use this field in place of a thanks-page designation "
            u"to determine final action after calling your action "
            u"adapter (if you have one). You would usually use "
            u"this for a custom success template or script. "
            u"Leave empty if unneeded. Otherwise, specify as you "
            u"would a CMFFormController action type and argument, "
            u"complete with type of action to execute "
            u'(e.g., "redirect_to" or "traverse_to") '
            u"and a TALES expression. For example, "
            u'"Redirect to" and "string:thanks-page" would redirect '
            u'to "thanks-page".',
        ),
        default=u"",
        constraint=isTALES,
        required=False,
    )
    directives.write_permission(formActionOverride=config.EDIT_TALES_PERMISSION)
    formActionOverride = zope.schema.TextLine(
        title=_(u"label_formactionoverride_text", default=u"Custom Form Action"),
        description=_(
            u"help_formactionoverride_text",
            default=u"Use this field to override the form action attribute. "
            u"Specify a URL to which the form will post. "
            u"This will bypass form validation, success action "
            u"adapter and thanks page.",
        ),
        default=u"",
        required=False,
        constraint=isTALES,
    )
    directives.write_permission(onDisplayOverride=config.EDIT_TALES_PERMISSION)
    onDisplayOverride = zope.schema.TextLine(
        title=_(u"label_OnDisplayOverride_text", default=u"Form Setup Script"),
        description=_(
            u"help_OnDisplayOverride_text",
            default=u"A TALES expression that will be called when the form is "
            u"displayed. Leave empty if unneeded. The most common "
            u"use of this field is to call a python script that "
            u"sets defaults for multiple fields by pre-populating "
            u"request.form. "
            u"Any value returned by the expression is ignored. "
            u"PLEASE NOTE: errors in the evaluation of this "
            u"expression will cause an error on form display.",
        ),
        constraint=isTALES,
        required=False,
        default=u"",
    )
    directives.write_permission(afterValidationOverride=config.EDIT_TALES_PERMISSION)
    afterValidationOverride = zope.schema.TextLine(
        title=_(
            u"label_AfterValidationOverride_text", default=u"After Validation Script"
        ),
        description=_(
            u"help_AfterValidationOverride_text",
            default=u"A TALES expression that will be called after the form is"
            u"successfully validated, but before calling an action "
            u"adapter (if any) or displaying a thanks page."
            u"Form input will be in the request.form dictionary."
            u"Leave empty if unneeded. The most "
            u"common use of this field is to call a python script"
            u"to clean up form input or to script an alternative "
            u"action. Any value returned by the expression is ignored."
            u"PLEASE NOTE: errors in the evaluation of this "
            u"expression willcause an error on form display.",
        ),
        constraint=isTALES,
        required=False,
        default=u"",
    )
    directives.write_permission(headerInjection=config.EDIT_TALES_PERMISSION)
    headerInjection = zope.schema.TextLine(
        title=_(u"label_headerInjection_text", default=u"Header Injection"),
        description=_(
            u"help_headerInjection_text",
            default=u"This override field allows you to insert content into "
            u"the xhtml head. The typical use is to add custom CSS "
            u"or JavaScript. Specify a TALES expression returning a "
            u"string. The string will be inserted with no "
            u"interpretation. PLEASE NOTE: errors in the evaluation "
            u"of this expression will cause an error on form display.",
        ),
        constraint=isTALES,
        required=False,
        default=u"",
    )
    directives.write_permission(submitLabelOverride=config.EDIT_TALES_PERMISSION)
    submitLabelOverride = zope.schema.TextLine(
        title=_(
            u"label_submitlabeloverride_text", default=u"Custom Submit Button Label"
        ),
        description=_(
            u"help_submitlabeloverride_text",
            default=u"This override field allows you to change submit button "
            u"label. The typical use is to set label with request "
            u"parameters. Specify a TALES expression returning a "
            u"string. PLEASE NOTE: errors in the evaluation of this "
            u"expression will cause an error on form display.",
        ),
        constraint=isTALES,
        required=False,
        default=u"",
    )


class IEasyFormImportFormSchema(Interface):
    """Schema for easyform import form."""

    upload = zope.schema.Bytes(title=PMF(u"Upload"), required=True)


class IEasyFormThanksPage(Interface):
    """Marker interface for thanks page."""


class IRenderWidget(Interface):
    """Marker for widget render BrowserView"""
